Attribute VB_Name = "ExecuteScriptModule"
Option Explicit

' this module holds all operations dealing with loading and executing the search scripts

Const cVarib = 14
Const cStar = 15
Const cChar = 16
Const cCharBack = 17
Const cStarBack = 18
Const cSpecificChar = 19

Const cMinSpecialChar = 14
Const cMaxSpecialChar = 26

Public Function LoadScript() As Boolean
    Dim fd As Integer
    Dim i As Integer
    Dim i2 As Integer
    Dim s As String
    Dim InputLine As Integer
    
ErrorTryAgain:
    InputLine = 0: ScriptLines = 0: CSL = 0
    fd = FreeFile()
    On Error GoTo LoadError
    Open App.Path & "\Weather Station.ini" For Input As #fd
    On Error GoTo 0
    
    While Not EOF(fd)
        Do
            If EOF(fd) Then GoTo ExitLoop
            InputLine = InputLine + 1
            On Error GoTo LoadError
            Line Input #fd, s
            On Error GoTo 0
            s = Trim$(s)
        Loop While Left$(s, 1) = ";" Or Len(s) = 0
        ' convert any tabs to spaces
        While InStr(s, vbTab) <> 0
            Mid$(s, InStr(s, vbTab), 1) = " "
        Wend
        s = Trim$(s)
        i = InStr(s, ":")
        If i <> 0 Then i2 = InStr(i + 1, s, ":")
        If i2 = 0 Then MsgBox "Missing "":"" character in: " & s, 16, App.FileDescription: Exit Function
        Script(ScriptLines).LineNbr = InputLine
        Script(ScriptLines).Operator = Trim$(UCase$(Left$(s, i - 1)))
        Script(ScriptLines).Switches = Mid$(s, i + 1, i2 - i - 1)
        Script(ScriptLines).Data = Trim$(Mid$(s, i2 + 1))
        ScriptLines = ScriptLines + 1
    Wend
    
ExitLoop:
    Close fd
    LoadScript = True
    Exit Function

LoadError:
    Close fd
    If MsgBox("Error reading from " + App.Path & "\Weather Station.ini" + vbCrLf + Err.Description + Format$(Err.Number, " (#)"), 277, App.FileDescription) = vbRetry Then
        GoTo ErrorTryAgain
    End If

End Function

Public Sub TriggerGetURL()
    Dim s As String
    
    ' skip any subsequent schedule lines
    While Script(CSL).Operator = "SCHEDULE"
        CSL = CSL + 1
        If CSL >= ScriptLines Then Err.Raise cScriptErr, , "Line" & Str(Script(CSL).LineNbr) & " - Expected ""URL:"""
    Wend
    
    If Script(CSL).Operator <> "URL" Then Err.Raise cScriptErr, , "Line" & Str(Script(CSL).LineNbr) & " - Expected ""URL:"" in " & s
    
    ' skip loading URL's for tabs that are hidden
    If Script(CSL).Switches = "a" And Main.TabBar.TabVisible(SatelliteTab) = False Then GoTo SkipThisURL
    If Script(CSL).Switches = "b" And Main.TabBar.TabVisible(SynopticTab) = False Then GoTo SkipThisURL
    
    s = Script(CSL).Data
    DebugLog "Getting URL: " & s
    CSL = CSL + 1
    
    ' we now have to unload/load the control to clear its cache (bug fix for Microsoft "feature")
    Unload Main.WB(1)
    Load Main.WB(1)
    Main.WB(1).Visible = True
    DoEvents
    
    ' now navigate to the correct page
    On Error GoTo ExecuteErr
    Main.WB(1).Navigate s
    On Error GoTo 0
    
    DoEvents
    Exit Sub

SkipThisURL:
    CSL = CSL + 1
    Exit Sub

    
ExecuteErr:
    DebugLog "Execute error: " & s & " Error = " & Str(Err.Number) & " (" & Err.Description & ")"
End Sub

Public Function CheckSchedule() As Boolean
    Dim CheckValid As Boolean
    Dim i As Integer

    If Script(CSL).Operator <> "SCHEDULE" Then Err.Raise cScriptErr, , "Line" & Str(Script(CSL).LineNbr) & " - Expected ""SCHEDULE:"""
    
    If TimeBeingScanned = "FullScan" Then
        GoTo GotSchedule
    Else
        While Script(CSL).Operator = "SCHEDULE"
            If Not ArgExtract(",", "", False, Script(CSL).Data) Then
                Err.Raise cScriptErr, , "Line" & Str(Script(CSL).LineNbr) & " - " & ArgError
            End If
            'Debug.Print "in CheckSchedule"
            For i = 0 To ArgC - 1
                On Error Resume Next
                CheckValid = (TimeValue(TimeBeingScanned) = TimeValue(ArgV(i)))
                If Err Then
                    On Error GoTo 0
                    Err.Raise cScriptErr, , "Line" & Str(Script(CSL).LineNbr) & " - Invalid time format: " & ArgV(i)
                End If
                On Error GoTo 0
                If CheckValid Then GoTo GotSchedule
            Next i
            CSL = CSL + 1
            If CSL >= ScriptLines Then Err.Raise cScriptErr, , "Line" & Str(Script(CSL).LineNbr) & " - Expected ""URL:"""
        Wend
    End If

    ' did not find a scheduled event
    CheckSchedule = False
    Exit Function
    
GotSchedule:
    ' skip any subsequent schedule lines
    While Script(CSL).Operator = "SCHEDULE"
        CSL = CSL + 1
        If CSL >= ScriptLines Then Err.Raise cScriptErr, , "Line" & Str(Script(CSL).LineNbr) & " - Expected ""URL:"""
    Wend
    
' set all variables to an error value
    For i = CSL To ScriptLines - 1
        If Script(i).Operator = "SCHEDULE" Then Exit For
        If Script(i).Operator <> "URL" Then SetVar Script(i).Operator, cInvalidValue
    Next i
    CheckSchedule = True
End Function


Public Function GetVariable(Dat As String, URL As String) As Boolean
    Dim s As String
    Dim t As String
    
    If CSL >= ScriptLines Then Exit Function
    If Script(CSL).Operator = "SCHEDULE" Or Script(CSL).Operator = "URL" Then Exit Function

    If GetVar(Script(CSL).Operator) = cInvalidValue Or GetVar(Script(CSL).Operator) = "-" Then
        If InStr(Script(CSL).Switches, "x") <> 0 Then
            s = Script(CSL).Data
        Else
            s = GetVariableData(Script(CSL).Data, Dat)
            ' if we are to prepend a url and if the string does not have a url
            If InStr(Script(CSL).Switches, "u") <> 0 And InStr(s, "://") = 0 Then
                If Left$(s, 1) = "/" Then
                    ' this is an absolute reference (within the site) so we just prepend the site
                    On Error Resume Next
                    s = Left$(URL, InStr(InStr(URL, "."), URL, "/") - 1) & s
                    If Err Then
                        On Error GoTo 0
                        Err.Raise cScriptErr, , "Line" & Str(Script(CSL).LineNbr) & " - Expected URL like http://xxx.xxx"
                    End If
                    On Error GoTo 0
                Else
                    ' this is relative to the current page so prepend the whole URL (without the page)
                    s = GetDirPath(URL) & "/" & s
                End If
            End If
        End If
            While InStr(Script(CSL).Switches, "c") = 0 And InStr(s, vbCrLf) > 0
                s = Left$(s, InStr(s, vbCrLf) - 1) & " " & Mid$(s, InStr(s, vbCrLf) + Len(vbCrLf))
            Wend
        If InStr(Script(CSL).Switches, "s") = 0 Then s = Trim$(s)
        SetVar Script(CSL).Operator, s
        DebugLog Script(CSL).Operator & " = " & s
    End If
    CSL = CSL + 1
    GetVariable = True
    
End Function

' get the data for a variable from the incomming data
' args: Script = variable search script with special tokens
'       dat = HTML page text
' returns the variable data as a string
Public Function GetVariableData(SearchScript As String, Dat As String) As String
    Dim i As Integer
    Dim i2 As Integer
    Dim p As Long
    Dim p2 As Long
    Dim s As String
    Dim s2 As String
    Dim st As String
    
    ' load the search string into a local variable (st) converting special characters as we go
    For i = 1 To Len(SearchScript)
        s = LCase$(Mid$(SearchScript, i, 1))
        If s = "*" Then
            st = st & Chr$(cStar)
        ElseIf s = "?" Then
            st = st & Chr$(cChar)
        ElseIf s = "~" Then
            st = st & Chr$(cSpecificChar)
        ElseIf s = "^" Then
            st = st & Chr$(cVarib)
        ElseIf s = "\" Then
            If i = Len(SearchScript) Then
                st = st & s
            Else
                i = i + 1
                s = LCase$(Mid$(SearchScript, i, 1))
                Select Case s
                    Case "t"
                        st = st & vbTab
                    Case "b"
                        st = st & Chr$(cStarBack)
                    Case "s"
                        st = st & " "
                    Case "n"
                        st = st & vbCrLf
                    Case Else
                        st = st & s
                End Select
            End If
        Else
            st = st & s
        End If
    Next i
    If Len(st) = 0 Then Err.Raise cScriptErr, , "Line" & Str(Script(CSL).LineNbr) & " - Script has zero length"
    
    ' step through the search string processing search tokens and the variable token
    p = 1
    i = 1
    While i <= Len(st)
        Select Case Mid$(st, i, 1)
            Case Chr$(cVarib)
                p2 = p
                i2 = i
                If Not ProcessStar(i, st, p, Dat) Then GoTo ErrVarNotFound
                s2 = Mid$(Dat, p2, p - p2 - (i - i2))
                'If s2 = "-" Then s2 = cInvalidValue
                GetVariableData = s2
                Exit Function
            Case Chr$(cStarBack)
                If Not ProcessStarBack(i, st, p, Dat) Then GoTo ErrVarNotFound
            Case Chr$(cStar)
                If Not ProcessStar(i, st, p, Dat) Then GoTo ErrVarNotFound
            Case Else
                Err.Raise cScriptErr, , "Line" & Str(Script(CSL).LineNbr) & " - Unexpected token: " & Mid$(st, i)
        End Select
        i = i + 1
    Wend
    Err.Raise cScriptErr, , "Line" & Str(Script(CSL).LineNbr) & " - Variable token not found"
    Exit Function
    
ErrVarNotFound:
    GetVariableData = cInvalidValue
End Function

' process a wildcard token (*) in the search string
' searches through the HTML data for the text following the wildcard token
' args: si = current pointer within the search script
'       st = the text of the search script
'       p = current pointer within the HTML data
'       dat = the HTML data
' returns:  si = pointing to the next token in the search text
'           p = pointing the end of the text just found
Public Function ProcessStar(ByRef si As Integer, st As String, ByRef p As Long, Dat As String) As Boolean
    Dim c As String
    Dim j As Integer

    si = si + 1                                 ' step over the wildcard character
    If Len(st) < si Then ' Special case of a wildcard at end of script
        p = Len(Dat)
        ProcessStar = True
        Exit Function
    End If
    For j = si To Len(st)
        If Len(st) < j Then Exit For
        If Mid$(st, j, 1) >= Chr$(cMinSpecialChar) And Mid$(st, j, 1) <= Chr$(cMaxSpecialChar) Then Exit For
    Next j
    If j = si Then Err.Raise cScriptErr, , "Unexpected token: " & Mid$(st, j)      ' error if the next char is a special char
    p = InStr(p, Dat, Mid$(st, si, j - si), vbTextCompare)
    If p = 0 Then Exit Function
    p = p + (j - si)
    si = j - 1
    ProcessStar = True
End Function

' process a backwards wildcard token (\s) in the search string
' searches backwards through the HTML data for the text following the wildcard token
' args: si = current pointer within the search script
'       st = the text of the search script
'       p = current pointer within the HTML data
'       dat = the HTML data
' returns:  si = pointing to the next token in the search text
'           p = pointing the end of the text just found
Public Function ProcessStarBack(ByRef si As Integer, st As String, ByRef p As Long, Dat As String) As Boolean
    Dim c As String
    Dim j As Integer
    Dim p2 As Long

    si = si + 1                                 ' step over the wildcard character
    If Len(st) < si Then ' Special case of a wildcard at end of script
        p = 1
        ProcessStarBack = True
        Exit Function
    End If
    For j = si To Len(st)
        If Len(st) < j Then Exit For
        If Mid$(st, j, 1) >= Chr$(cMinSpecialChar) And Mid$(st, j, 1) <= Chr$(cMaxSpecialChar) Then Exit For
    Next j
    If j = si Then Err.Raise cScriptErr, , "Unexpected token: " & Mid$(st, j)      ' error if the next char is a special char
    
    For p2 = p - 1 To 0 Step -1
        If p2 <= 0 Then Exit Function
        If InStr(p2, Dat, Mid$(st, si, j - si), vbTextCompare) = p2 Then Exit For
    Next p2
    p = p2 + (j - si)
    si = j - 1
    ProcessStarBack = True
End Function

