Attribute VB_Name = "GraphModule"
' this value is recorded in the second line of WeatherStation.dat file
' it should be incremented if the format of the file is changed
' a mismatch in version numbers indicates that the file should be discarded or converted
Const DatFileVersion = "4"      ' note that this does not have a conversion from the previous version (which was 3)

Type HistoryRecord
    DayTime As Date
    Temp As Single
    Prs As Single
    Rain As Single
    WindD As Integer
    WindS As Integer
    Humid As Integer
End Type
Const MaxHist = 650
Dim Hist(0 To MaxHist) As HistoryRecord
Dim HistCnt As Integer

' constants representing the number of days to display when radio button Main.rTimeScale(?).Value is selected
Const NbrDays0 = 1
Const NbrDays1 = 2
Const NbrDays2 = 4

' constants representing the radio button controlling the parameter to display
Const rTemp = 0
Const rHumid = 1
Const rRain = 2
Const rPrs = 3
Const rWindD = 4
Const rWindS = 5



Public Sub LoadHist()
    Dim fd As Integer
    Dim i As Integer
    Dim s As String
    
    fd = FreeFile()
    On Error GoTo LoadError
    Open App.Path & "\Weather Station.dat" For Input As #fd
    
    Line Input #fd, s        ' load and discard the title line
    Line Input #fd, s        ' load the version number
    
    ' use the SelectCase to load (and convert) previous data file formats
    Select Case s
        Case DatFileVersion     ' this is the current version
            While Not EOF(fd) And HistCnt < MaxHist
                Input #fd, s, Hist(HistCnt).Temp, Hist(HistCnt).Prs, Hist(HistCnt).Rain, Hist(HistCnt).WindD, Hist(HistCnt).WindS, Hist(HistCnt).Humid
                Hist(HistCnt).DayTime = CDate(s)
                HistCnt = HistCnt + 1
            Wend
        Case 2
            While Not EOF(fd) And HistCnt < MaxHist
                Input #fd, s, Hist(HistCnt).Temp, Hist(HistCnt).Prs, Hist(HistCnt).Rain, Hist(HistCnt).WindD, Hist(HistCnt).WindS
                Hist(HistCnt).DayTime = CDate(s)
                HistCnt = HistCnt + 1
            Wend
        Case 1
            While Not EOF(fd) And HistCnt < MaxHist
                Input #fd, s, Hist(HistCnt).Temp, Hist(HistCnt).Prs, Hist(HistCnt).Rain, Hist(HistCnt).WindD
                Hist(HistCnt).DayTime = CDate(s)
                HistCnt = HistCnt + 1
            Wend
        Case Else
            GoTo LoadError          ' load blank data if the version number is not recognised
    End Select
    
    On Error GoTo 0
    
ExitLoop:
    Close fd
    Exit Sub

LoadError:
    Close fd
    For i = 0 To MaxHist
        Hist(i).DayTime = 0        ' flag all history as invalid
    Next i

End Sub


Public Sub AddToGraph(DayTime As String, Temp As String, Prs As String, Rain As String, WindD As String, WindS As String, Humid As String)
    Dim fd As Integer
    Dim i As Integer
    
    Dim DT As Date
    Dim t As Single
    Dim p As Single
    Dim r As Single
    Dim WD As Integer
    Dim WS As Integer
    Dim h As Integer

    ' extract the various measurements
    ' there is a lot of error checking the make sure that invalid data id not recorded
    If DayTime = cInvalidValue Then Exit Sub
    If Temp = cInvalidValue Then Exit Sub
    If Prs = cInvalidValue Then Exit Sub
    If Rain = cInvalidValue Then Exit Sub
    If WindD = cInvalidValue Then Exit Sub
    If WindS = cInvalidValue Then Exit Sub
    If Humid = cInvalidValue Then Exit Sub
    
    On Error GoTo AbortAddToGraph   ' any conversion errors will cause us to skip further processing
    DT = CDate(DayTime)             ' this result is not used, it just checks for errors in DayTime
    t = CSng(Temp)
    p = CSng(Prs)
    r = CSng(Rain)
    WS = CInt(WindS)
    h = CInt(Humid)
    On Error Resume Next
    
    ' add the date to the DayTime
    ' rather complicated because the computer's clock could be in front or behind the BOM clock
    ' and an error will show up around midnight
    If Hour(DT) < 6 And Hour(Now) > 18 Then
        DT = CDate(Format$(Now + 1, "d-mmm-yyyy ") & DayTime)
    ElseIf Hour(DT) > 18 And Hour(Now) < 6 Then
        DT = CDate(Format$(Now - 1, "d-mmm-yyyy ") & DayTime)
    Else
        DT = CDate(Format$(Now, "d-mmm-yyyy ") & DayTime)
    End If
    
    If DT = Hist(0).DayTime Then DisplayGraph       ' this forces the graph current readings to be displayed
    
    ' more sanity checking
    If DT <= Hist(0).DayTime Then Exit Sub          ' exit if we already have recorded this reading
    If DT < Now - 1 Then Exit Sub                   ' exit if this reading is more than one day old
    If t < -5 Or t > 50 Then Exit Sub               ' exit if the temperature reading is extreme
    If p < 900 Or p > 1200 Then Exit Sub            ' exit if the pressure reading is extreme
    If WS < 0 Or WS > 200 Then Exit Sub             ' exit if the wind speed reading is extreme
    If h < 0 Or h > 100 Then Exit Sub               ' exit if the humidity reading is extreme
    ' end of error checking
    
    ' convert the wind direction to an integer
    For i = 0 To 16
        If WindD = Images.WindDirPic(i).Tag Then WD = 16 - i
    Next i

    ' shift the history array up by one
    For i = MaxHist To 1 Step -1
        Hist(i) = Hist(i - 1)
    Next i
    
    Hist(0).DayTime = DT
    Hist(0).Temp = t
    Hist(0).Prs = p
    Hist(0).Rain = r
    Hist(0).WindD = WD
    Hist(0).WindS = WS
    Hist(0).Humid = h
       
    fd = FreeFile()
    On Error GoTo WriteError
    Open App.Path & "\Weather Station.dat" For Output As #fd
    Print #fd, "Weather Station history data - DO NOT EDIT THIS FILE"
    Print #fd, DatFileVersion
    For i = 0 To MaxHist
        Write #fd, Format$(Hist(i).DayTime, "d-mmm-yyyy hh:mm am/pm"), Hist(i).Temp, Hist(i).Prs, Hist(i).Rain, Hist(i).WindD, Hist(i).WindS, Hist(i).Humid
    Next i
    
    Close fd
    ValidGraphData = True
    DisplayGraph
    
AbortAddToGraph:
    Exit Sub

WriteError:
    Close fd
    If MsgBox("Error writing to " + App.Path & "\Weather Station.dat" + vbCrLf + Err.Description + Format$(Err.Number, " (#)"), 277, App.FileDescription) = vbRetry Then
        Resume
    End If

End Sub


Public Sub DisplayGraph()
    Dim i As Integer
    Dim MaxV As Single
    Dim MinV As Single
    Dim r1 As Single
    Dim r2 As Single
    Dim ValidGraphData As Boolean
    
    Main.Graph.Cls
    
    If Main.rGraph(rTemp).Value Then
        'scan through the history to find the max and min temps and set the scale accordingly
        MinV = 10
        MaxV = 20
        For i = 0 To MaxHist
            If Hist(i).DayTime < Now - NbrDays2 Then Exit For
            If Hist(i).DayTime <> 0 Then
                If Hist(i).Temp > MaxV Then MaxV = Hist(i).Temp
                If Hist(i).Temp < MinV Then MinV = Hist(i).Temp
            End If
            If Hist(i).DayTime < Now - NbrDays2 Then Exit For
        Next i
        SetupVScale Int(MinV / 10) * 10, Int((MaxV + 9.9) / 10) * 10, 10, 2, False
        Main.Units.Caption = "C"
    
    ElseIf Main.rGraph(rPrs).Value Then
        'scan through the history to find the max and min pressure and set the scale accordingly
        MinV = 1000
        MaxV = 1030
        For i = 0 To MaxHist
            If Hist(i).DayTime < Now - NbrDays2 Then Exit For
            If Hist(i).DayTime <> 0 Then
                If Hist(i).Prs > MaxV Then MaxV = Hist(i).Prs
                If Hist(i).Prs < MinV Then MinV = Hist(i).Prs
            End If
            If Hist(i).DayTime < Now - NbrDays2 Then Exit For
        Next i
        SetupVScale Int(MinV / 5) * 5, Int((MaxV + 4.99) / 5) * 5, 10, 2, False
        Main.Units.Caption = "hPa"
    ElseIf Main.rGraph(rWindS).Value Then
        'scan through the history to find the max wind speed and set the scale accordingly
        MaxV = 30
        For i = 0 To MaxHist
            If Hist(i).DayTime < Now - NbrDays2 Then Exit For
            If Hist(i).DayTime <> 0 And Hist(i).WindS > MaxV Then MaxV = Hist(i).WindS
        Next i
        SetupVScale Int(MinV / 10) * 10, Int((MaxV + 9) / 10) * 10, 10, 2, False
        Main.Units.Caption = "Km/h"
    ElseIf Main.rGraph(rWindD).Value Then
        SetupVScale 0, 16, 4, 1, True
        Main.Units.Caption = ""
    ElseIf Main.rGraph(rHumid).Value Then
        SetupVScale 0, 100, 25, 5, False
        Main.Units.Caption = "% RH"
    Else
        'scan through the history to find the max rainfall and set the scale accordingly
        MaxV = 4
        For i = 0 To MaxHist
            If Hist(i).DayTime <> 0 And RainRate(i) > MaxV Then MaxV = RainRate(i)
            If Main.rTimeScale(0).Value And Hist(i).DayTime < Now - NbrDays0 Then Exit For
            If Main.rTimeScale(1).Value And Hist(i).DayTime < Now - NbrDays1 Then Exit For
            If Main.rTimeScale(2).Value And Hist(i).DayTime < Now - NbrDays2 Then Exit For
        Next i
        If Int(MaxV + 0.9999) <= 8 Then
            SetupVScale 0, Int((MaxV + 3.9999) / 4) * 4, 1, 0.2, False
        Else
            SetupVScale 0, Int((MaxV + 3.9999) / 4) * 4, 4, 1, False
        End If
        Main.Units.Caption = "mm/hr"
    End If
    
    If Main.rTimeScale(0).Value Then
        SetupHScale CDbl(Now - NbrDays0), CDbl(Now)
    ElseIf Main.rTimeScale(1).Value Then
        SetupHScale CDbl(Now - NbrDays1), CDbl(Now)
    Else
        SetupHScale CDbl(Now - NbrDays2), CDbl(Now)
    End If

    For i = HistCnt To 1 Step -1
        If Hist(i - 1).DayTime - Hist(i).DayTime < 1 / 24 And Hist(i).DayTime > CDate(Main.Graph.ScaleLeft * 0.9) Then
            ValidGraphData = True
            If Main.rGraph(rTemp).Value Then Main.Graph.Line (Hist(i).DayTime, Hist(i).Temp)-(Hist(i - 1).DayTime, Hist(i - 1).Temp)
            If Main.rGraph(rPrs).Value Then Main.Graph.Line (Hist(i).DayTime, Hist(i).Prs)-(Hist(i - 1).DayTime, Hist(i - 1).Prs)
            If Main.rGraph(rRain).Value Then Main.Graph.Line (Hist(i).DayTime, RainRate(i))-(Hist(i - 1).DayTime, RainRate(i - 1))
            If Main.rGraph(rWindD).Value And Hist(i).DayTime <> 0 Then
                If Abs(Hist(i - 1).WindD - Hist(i).WindD) < 14 And Hist(i - 1).DayTime <> 0 Then
                    Main.Graph.Line (Hist(i).DayTime, Hist(i).WindD)-(Hist(i - 1).DayTime, Hist(i - 1).WindD)
                Else
                    'Main.Graph.Line (Hist(i).DayTime, Hist(i).WindD)-(Hist(i - 1).DayTime, Hist(i - 1).WindD), &H80000010
                    Main.Graph.Line (Hist(i).DayTime, Hist(i - 1).WindD)-(Hist(i - 1).DayTime, Hist(i - 1).WindD)
                End If
            End If
            If Main.rGraph(rWindS).Value Then Main.Graph.Line (Hist(i).DayTime, Hist(i).WindS)-(Hist(i - 1).DayTime, Hist(i - 1).WindS)
            If Main.rGraph(rHumid).Value Then Main.Graph.Line (Hist(i).DayTime, Hist(i).Humid)-(Hist(i - 1).DayTime, Hist(i - 1).Humid)
        End If
    Next i
    
    If Not ValidGraphData Then
        ' display a helpful message stating that no data is available for display
        Main.Graph.CurrentY = Main.Graph.ScaleTop + Main.Graph.TextHeight("AA") / 3
        Main.Graph.CurrentX = Main.Graph.ScaleLeft
        Main.Graph.Print "   NO HISTORY - LET PROGRAM RUN FOR A NUMBER OF HOURS"
    Else
        ' print the current value
        Main.CurrentValue.Cls
        Main.CurrentValue.CurrentY = Main.Graph.CurrentY - Main.CurrentValue.TextHeight("99") / 2
        If Main.CurrentValue.CurrentY > Main.CurrentValue.ScaleTop Then Main.CurrentValue.CurrentY = Main.CurrentValue.ScaleTop
        If Main.CurrentValue.CurrentY + Main.CurrentValue.TextHeight("99") < Main.CurrentValue.ScaleTop + Main.CurrentValue.ScaleHeight Then Main.CurrentValue.CurrentY = Main.CurrentValue.CurrentY - Main.CurrentValue.TextHeight("99") / 2
        If Main.rGraph(rTemp).Value And GetVar("C-TEMP") <> cInvalidValue Then Main.CurrentValue.Print GetVar("C-TEMP") & "C"
        If Main.rGraph(rPrs).Value And GetVar("C-BAROMETER") <> cInvalidValue Then Main.CurrentValue.Print GetVar("C-BAROMETER")
        If Main.rGraph(rWindD).Value And GetVar("C-WIND-DIR") <> cInvalidValue Then Main.CurrentValue.Print " " & GetVar("C-WIND-DIR")
        If Main.rGraph(rRain).Value And RainRate(0) > 0 Then Main.CurrentValue.Print Format$(RainRate(0), "0.0")
        If Main.rGraph(rWindS).Value And GetVar("C-WIND-SPEED") <> cInvalidValue Then Main.CurrentValue.Print GetVar("C-WIND-SPEED") & " Kph"
        If Main.rGraph(rHumid).Value And GetVar("C-HUMID") <> cInvalidValue Then Main.CurrentValue.Print " " & GetVar("C-HUMID") & "%"
    End If

End Sub

Private Sub SetupHScale(LeftValue As Double, RightValue As Double)
    Dim i As Long
    Dim LastMidnight As Single
    Dim LastMidday As Single
    Dim PrintedFirstDay As Boolean
    
    Main.HScale.Cls
    
    Main.Graph.ScaleLeft = LeftValue
    Main.Graph.ScaleWidth = RightValue - LeftValue
    Main.HScale.ScaleLeft = LeftValue
    Main.HScale.ScaleWidth = RightValue - LeftValue

    Main.HScale.Line (LeftValue, 0)-(RightValue, 0)
    
    ' loop through the hours making the appropiate tick marks on the horizontal scale
    ' most of the code is with associated with placement of the day lables
    For i = Int(LeftValue * 24) To Int(RightValue * 24)
        'If Main.HScale.ScaleWidth > 3 Then
        '    If i Mod 2 = 0 Then Main.HScale.Line (i / 24, 0)-(i / 24, 75) ' mark a minor hour
        'Else
            Main.HScale.Line (i / 24, 0)-(i / 24, 75)                    ' mark a minor hour
        'End If
        If i Mod 6 = 0 Then Main.HScale.Line (i / 24, 0)-(i / 24, 125)  ' mark 6am or 6pm
        
        If i Mod 24 = 12 Then                                           ' if this is midday
            Main.HScale.Line (i / 24, 0)-(i / 24, 150)                  ' mark midday
            LastMidday = i / 24                                         ' and record its value for later
            If Format$(i / 24, "dddd") <> Format$(Now, "dddd") Then     ' if the day is NOT today
                ' the day lable is centered around the 12 noon mark
                Main.HScale.CurrentX = Main.HScale.CurrentX - Main.HScale.TextWidth(Format$(i / 24, "dddd")) / 2
                Main.HScale.CurrentY = 150
                ' only print the day label if it fits completely onto the scale
                ' code further down will handle the case if it does not fit
                If Main.HScale.CurrentX >= Main.HScale.ScaleLeft Then
                    Main.HScale.Print Format$(i / 24, "dddd")
                    PrintedFirstDay = True                              ' show that we did print it
                End If
            End If
        End If
        
        If i Mod 24 = 0 Then                                            ' if this is midnight
            Main.HScale.Line (i / 24, 0)-(i / 24, 300)                  ' mark midnight
            LastMidnight = i / 24                                       ' and record its value for later
            ' if we have not already printed the previous day label AND we have space to fit it in THEN print it
            If Not PrintedFirstDay And (Main.HScale.TextWidth(Format$((i - 1) / 24, " dddd ")) < i / 24 - Main.HScale.ScaleLeft) Then
                Main.HScale.CurrentX = Main.HScale.ScaleLeft            ' justify against the left margin
                Main.HScale.CurrentY = 150
                Main.HScale.Print Format$((i - 1) / 24, "dddd")
                PrintedFirstDay = True                                  ' show that we did print it
            End If
        End If
    Next i
    
    ' this code handles the printing of the "Today" label
    ' firstly, if we can centre it around the last midday tick mark then do so
    If LastMidday > LastMidnight And LastMidday + Main.HScale.TextWidth(" Today ") / 2 < RightValue Then
        Main.HScale.CurrentX = LastMidday - Main.HScale.TextWidth(" Today ") / 2
        Main.HScale.CurrentY = 150
        Main.HScale.Print " Today "
    ' otherwise, if we can fit it in by right justifying it - then do so
    ' note that this might not be possible and in that case the "Today" label is not displayed at all
    ElseIf Main.HScale.TextWidth(" Today ") < (RightValue - LastMidnight) Then
        Main.HScale.CurrentX = RightValue - Main.HScale.TextWidth(" Today ")
        Main.HScale.CurrentY = 150
        Main.HScale.Print " Today "
    End If
    
End Sub


Private Sub SetupVScale(BottomValue As Single, TopValue As Single, MajorTicMark As Single, MinorTicMark As Single, WindD As Boolean)
    Const Oversize = 0.075          ' amount to oversize the vert scale picture box as a fraction
    Static DoneOversize As Boolean
    Dim a As Single
    Dim s As String
    
    Main.VScale.Cls
    
    If Not DoneOversize Then
        Main.VScale.BorderStyle = 0
        Main.HScale.BorderStyle = 0
        Main.Graph.BorderStyle = 0
        Main.CurrentValue.BorderStyle = 0
        Main.Units.BorderStyle = 0
        a = Main.VScale.Height * Oversize
        Main.VScale.Height = Main.VScale.Height * (1 + Oversize * 2)
        Main.VScale.Top = Main.VScale.Top - a
        DoneOversize = True
    End If
    
    Main.Graph.ScaleTop = TopValue
    Main.Graph.ScaleHeight = BottomValue - TopValue
    Main.CurrentValue.ScaleTop = TopValue
    Main.CurrentValue.ScaleHeight = BottomValue - TopValue
    Main.VScale.ScaleHeight = (BottomValue - TopValue) * (1 + Oversize * 2)
    Main.VScale.ScaleTop = TopValue + ((TopValue - BottomValue) * Oversize)
    
    Main.VScale.Line (Main.VScale.ScaleWidth - 50, BottomValue)-(Main.VScale.ScaleWidth - 50, TopValue)
    
    For a = BottomValue To TopValue Step MinorTicMark
        Main.VScale.Line (Main.VScale.ScaleWidth - 75, a)-(Main.VScale.ScaleWidth - 50, a)
    Next a
    
    Main.Graph.DrawStyle = vbDot
    For a = Int(BottomValue / MajorTicMark) * MajorTicMark To TopValue Step MajorTicMark
        Main.Graph.Line (Main.Graph.ScaleLeft, a)-(Main.Graph.ScaleLeft + Main.Graph.ScaleWidth, a), &H80000010
        ' kludge alert
        ' if WindD = true then this is wind direction therefor define the label differently
        If WindD Then
            If a <> 0 Then Main.VScale.Line (Main.VScale.ScaleWidth - 150, a)-(Main.VScale.ScaleWidth - 50, a)
            s = Images.WindDirPic(16 - a).Tag
        Else
            Main.VScale.Line (Main.VScale.ScaleWidth - 150, a)-(Main.VScale.ScaleWidth - 50, a)
            s = Format$(a)
        End If
        Main.VScale.CurrentX = Main.VScale.ScaleWidth - 200 - Main.VScale.TextWidth(s)
        Main.VScale.CurrentY = Main.VScale.CurrentY - Main.VScale.TextHeight(s) / 2
        Main.VScale.Print s
    Next a
    Main.Graph.DrawStyle = vbSolid

End Sub

Public Sub ProcessMouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    Dim i As Integer
    Dim xx As Date
    
    xx = CDate(X)
    
    For i = 1 To HistCnt
        'Debug.Print i, CDate(X), Hist(i).DayTime, Hist(i + 1).DayTime
        If xx < Hist(i - 1).DayTime And xx > Hist(i).DayTime Then
            If Hist(i - 1).DayTime = 0 Or Hist(i).DayTime = 0 Or Hist(i - 1).DayTime - Hist(i).DayTime > 1 / 24 Then
                Exit For
            Else
                If Main.rGraph(rTemp).Value Then Main.Graph.ToolTipText = Format$(Hist(i).Temp, "0.0") & "C at " & Format$(Hist(i).DayTime, "h:mm am/pm dddd")
                If Main.rGraph(rPrs).Value Then Main.Graph.ToolTipText = Format$(Hist(i).Prs) & " hPa at " & Format$(Hist(i).DayTime, "h:mm am/pm dddd")
                If Main.rGraph(rWindD).Value Then Main.Graph.ToolTipText = Images.WindDirPic(16 - Hist(i).WindD).Tag & " at " & Format$(Hist(i).DayTime, "h:mm am/pm dddd")
                If Main.rGraph(rRain).Value Then Main.Graph.ToolTipText = Format$(RainRate(i), "0.0") & " mm/hour at " & Format$(Hist(i).DayTime, "h:mm am/pm dddd")
                If Main.rGraph(rWindS).Value Then Main.Graph.ToolTipText = Format$(Hist(i).WindS) & " Km/h at " & Format$(Hist(i).DayTime, "h:mm am/pm dddd")
                If Main.rGraph(rHumid).Value Then Main.Graph.ToolTipText = Format$(Hist(i).Humid) & "% at " & Format$(Hist(i).DayTime, "h:mm am/pm dddd")
                Exit Sub
            End If
        End If
    Next i
    Main.Graph.ToolTipText = "No data"
        
End Sub

' calculate the rate of rainfall given an index to the Hist() array
Private Function RainRate(Index As Integer) As Single
    Dim r As Single
    Dim t As Double
    
    If Index >= HistCnt Then Exit Function
    If Hist(Index).DayTime = 0 Or Hist(Index + 1).DayTime = 0 Then Exit Function
    
    r = Hist(Index).Rain - Hist(Index + 1).Rain         ' the amount of rainfall in the last period
    t = Hist(Index).DayTime - Hist(Index + 1).DayTime   ' timespan of the last period
    
    ' adjust if the sample period spans 9AM when the rainfall counter is rset to zero
    If r < 0 Or (Hour(Hist(Index + 1).DayTime) < 9 And Hour(Hist(Index).DayTime) >= 9) Then
        r = Hist(Index).Rain
        t = Hist(Index).DayTime - CDate(Format$(Hist(Index).DayTime, "dd-mmmm-yyyy") & " 9:00 AM")
    End If
    
    If t > 0 Then
        r = r / (t * 24)    ' calculate the rate of rainfall for the past sampling period
    Else
        r = 0               ' avoid divide by zero error - can happen if the sample time is exactly 9AM
    End If
    
    If r < 0 Or r > 100 Then
        r = 0   ' MsgBox "Invalid rain rate calculation" & vbCrLf & "r - " & Format$(r)
    End If
    
    RainRate = r
    
End Function
